# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['py2sfn_task_tools']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=2.2.1,<3.0.0',
 'boto3>=1.24.59,<2.0.0',
 'sfn-workflow-client>=1.1.1,<2.0.0']

setup_kwargs = {
    'name': 'py2sfn-task-tools',
    'version': '1.0.2',
    'description': 'Tools for tasks embedded in an AWS Step Functions state machine. This is a helper library for py2sfn.',
    'long_description': '# py2sfn-task-tools\n\n[![](https://img.shields.io/pypi/v/py2sfn-task-tools.svg)](https://pypi.org/pypi/py2sfn-task-tools/) [![License](https://img.shields.io/badge/License-BSD%203--Clause-blue.svg)](https://opensource.org/licenses/BSD-3-Clause)\n\nTools for tasks embedded in an [AWS Step Functions](https://aws.amazon.com/step-functions/) state machine. This is a helper library for [py2sfn](https://github.com/NarrativeScience/py2sfn).\n\nFeatures:\n\n- Offload state data to DynamoDB/S3 instead of storing data in the *very* constrained state machine input data object\n- Cancel the currently executing workflow\n\nTable of Contents:\n\n- [Installation](#installation)\n- [Guide](#guide)\n  - [Stopping the execution](#stopping-the-execution)\n  - [Working with the State Data Client](#working-with-the-state-data-client)\n- [Development](#development)\n\n## Installation\n\npy2sfn-task-tools requires Python 3.6 or above. It should be installed in a [py2sfn task entry point](https://github.com/NarrativeScience/py2sfn#task-entry-points).\n\n```bash\npip install py2sfn-task-tools\n```\n\n## Guide\n\nOnce the py2sfn-task-tools library is installed, a `Context` should be created and passed to the tasks. Each py2sfn task will then have a `context` object to work with.\n\n### Stopping the execution\n\nIf you need to stop/cancel/abort the execution from within a task, you can use the  `context.stop_execution` method within your task\'s `run` method. A common use case is if you need to check the value of a feature flag at the beginning of the execution and abort if it\'s false. For example:\n\n```python\nif not some_condition:\n    return await context.stop_execution()\n```\n\nYou can provide extra detail by passing `error` and `cause` keyword arguments to the `stop_execution` method. The `error` is a short string like a code or enum value whereas `cause` is a longer description.\n\n### Working with the State Data Client\n\nOne of the stated Step Functions best practices is to avoid passing large payloads between states; the input data limit is only 32K characters. To get around this, you can choose to store data from your task code in a DynamoDB table. With DynamoDB, we have an item limit of 400KB to work with. When you put items into the table you receive a pointer to the DynamoDB item which you can return from your task so it gets includes in the input data object. From there, since the pointer is in the `data` dict, you can reload the stored data in a downstream task. This library\'s `StateDataClient` class provides methods for putting and getting items from this DynamoDB table. It\'s available in your task\'s `run` method as `context.state_data_client`.\n\nThe client methods are split between "local" and "global" variants. Local methods operate on items stored within the project whereas global methods can operate on items that were stored from any project. Global methods require a fully-specified partition key (primary key, contains the execution ID) and table name to locate the item whereas local methods only need a simple key because the partition key and table name can be infered from the project automatically. The `put_*` methods return a dict with metadata about the location of the item, including the `key`, `partition_key`, and `table_name`. If you return this metadata object from a task, it will get put on the `data` object and you can call a `get_*` method later in the state machine.\n\nMany methods also accept an optional `index` argument. This argument needs to be provided when getting/putting an item that was originally stored as part of a `put_items` or `put_global_items` call. Providing the `index` is usually only done within a map iteration task.\n\nBelow are a few of the more common methods:\n\n#### `put_item`/`put_items`\n\nThe `put_item` method puts an item in the state store. It takes `key`, `data`, and `index` arguments. For example:\n\n```python\ncontext.state_data_client.put_item("characters", {"name": "jerry"})\ncontext.state_data_client.put_item("characters", {"name": "elaine"}, index=24)\n```\n\nNote that the item at the given array index doesn\'t actually have to exist in the table before you call `put_item`. However, if it doesn\'t exist then you may have a fan-out logic bug upstream in your state machine.\n\nThe `put_items` method puts an entire list of items into the state store. Each item will be stored separately under its corresponding array index. For example:\n\n```python\ncontext.state_data_client.put_items("characters", [{"name": "jerry"}, {"name": "elaine"}])\n```\n\n#### `get_item`\n\nThe `get_item` method gets the data attribute from an item in the state store. It takes `key` and `index` arguments. For example:\n\n```python\ncontext.state_data_client.get_item("characters")  # -> {"name": "jerry"}\ncontext.state_data_client.get_item("characters", index=24)  # -> {"name": "elaine"}\n```\n\n#### `get_item_for_map_iteration`/`get_global_item_for_map_iteration`\n\nThe `get_item_for_map_iteration` method gets the data attribute from an item in the state store using the `event` object. This method only works when called within a map iterator task. For example, if the `put_items` example above was called in a task, and its value was given to a map state to fan out, we can use the `get_item_for_map_iteration` method within our iterator task to fetch each item:\n\n```python\n# Iteration 0:\ncontext.state_data_client.get_item_for_map_iteration(event)  # -> {"name": "jerry"}\n# Iteration 1:\ncontext.state_data_client.get_item_for_map_iteration(event)  # -> {"name": "elaine"}\n```\n\nThis works because the map iterator state machine receives an input data object with the schema:\n\n```json\n{\n  "items_result_table_name": "<DynamoDB table for the project>",\n  "items_result_partition_key": "<execution ID>:characters",\n  "items_result_key": "characters",\n  "context_index": "<array index>",\n  "context_value.$": "1"\n}\n```\n\nThe `get_item_for_map_iteration` is a helper method that uses that input to locate the right item. The `get_global_item_for_map_iteration` method has the same signature. It should be called when you know that the array used to fan out could have come from another project (e.g. the map state is the first state in a state machine triggered by a subscription).\n\n## Development\n\nTo run functional tests, you need an AWS IAM account with permissions to:\n\n- Create/update/delete a DynamoDB table\n- Create/update/delete an S3 bucket\n\nSet the following environment variables:\n\n- `AWS_ACCESS_KEY_ID`\n- `AWS_SECRET_ACCESS_KEY`\n- `AWS_DEFAULT_REGION`\n\nTo run tests:\n\n```bash\ntox\n```\n',
    'author': 'Jonathan Drake',
    'author_email': 'jdrake@narrativescience.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/NarrativeScience/py2sfn-task-tools',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)
